@@ -37,6 +37,7 @@
 #include "icn.h"
 #include "image.h"
 #include "interface_base.h"
+#include "interface_gamearea.h"
 #include "localevent.h"
 #include "maps_tiles.h"
 #include "monster.h"
@@ -1271,7 +1272,9 @@ namespace Interface
             }
         }
         _setCursor();
-        _interface.updateCursor( 0 );
+
+        // The cursor must be set according to the position of mouse cursor.
+        _interface.updateCursor( _interface.getGameArea().GetValidTileIdFromPoint( LocalEvent::Get().getMouseCursorPos() ) );
     }
 
     void EditorPanel::getTownObjectProperties( int32_t & type, int32_t & color ) const
