@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2011 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -28,6 +28,7 @@
 #include <cassert>
 #include <cstddef>
 #include <sstream>
+#include <type_traits>
 
 #include "castle.h"
 #include "game.h"
@@ -36,7 +37,6 @@
 #include "logging.h"
 #include "maps.h"
 #include "maps_fileinfo.h"
-#include "race.h"
 #include "rand.h"
 #include "save_format_version.h"
 #include "serialize.h"
@@ -46,7 +46,7 @@
 namespace
 {
     std::array<Player *, maxNumOfPlayers + 1> playersArray{};
-    int humanColors{ Color::NONE };
+    PlayerColorsSet humanColors{ 0 };
 
     enum
     {
@@ -107,53 +107,33 @@ namespace
     }
 }
 
-bool Control::isControlAI() const
+Player::Player( const PlayerColor color /* = PlayerColor::NONE */ )
+    : _aiPersonality( AI::getRandomPersonality() )
+    , _color( color )
+    , _friendsColors( static_cast<PlayerColorsSet>( color ) )
 {
-    return ( CONTROL_AI & GetControl() ) != 0;
-}
-
-bool Control::isControlHuman() const
-{
-    return ( CONTROL_HUMAN & GetControl() ) != 0;
-}
-
-Player::Player( const int col /* = Color::NONE */ )
-    : control( CONTROL_NONE )
-    , color( col )
-    , race( Race::NONE )
-    , friends( col )
-    , _aiPersonality( AI::getRandomPersonality() )
-    , _handicapStatus( HandicapStatus::NONE )
-#if defined( WITH_DEBUG )
-    , _isAIAutoControlMode( false )
-    , _isAIAutoControlModePlanned( false )
-#endif
-{}
-
-std::string Player::GetDefaultName() const
-{
-    return Color::String( color );
+    // Do nothing.
 }
 
 std::string Player::GetName() const
 {
-    if ( name.empty() ) {
+    if ( _name.empty() ) {
         return GetDefaultName();
     }
 
-    return name;
+    return _name;
 }
 
 int Player::GetControl() const
 {
 #if defined( WITH_DEBUG )
     if ( _isAIAutoControlMode ) {
-        assert( ( control & CONTROL_HUMAN ) == CONTROL_HUMAN );
+        assert( ( _control & CONTROL_HUMAN ) == CONTROL_HUMAN );
         return CONTROL_AI;
     }
 #endif
 
-    return control;
+    return _control;
 }
 
 std::string Player::GetPersonalityString() const
@@ -166,13 +146,13 @@ bool Player::isPlay() const
     return Modes( ST_INGAME );
 }
 
-void Player::SetName( const std::string & newName )
+void Player::SetName( std::string newName )
 {
     if ( newName == GetDefaultName() ) {
-        name.clear();
+        _name.clear();
     }
     else {
-        name = newName;
+        _name = std::move( newName );
     }
 }
 
@@ -193,7 +173,7 @@ void Player::setHandicapStatus( const HandicapStatus status )
         return;
     }
 
-    assert( !( control & CONTROL_AI ) );
+    assert( !( _control & CONTROL_AI ) );
 
     _handicapStatus = status;
 }
@@ -201,7 +181,7 @@ void Player::setHandicapStatus( const HandicapStatus status )
 #if defined( WITH_DEBUG )
 void Player::setAIAutoControlMode( const bool enable )
 {
-    assert( ( control & CONTROL_HUMAN ) == CONTROL_HUMAN );
+    assert( ( _control & CONTROL_HUMAN ) == CONTROL_HUMAN );
 
     // If this mode should be enabled, then it happens immediately
     if ( enable ) {
@@ -217,7 +197,7 @@ void Player::setAIAutoControlMode( const bool enable )
 
 void Player::commitAIAutoControlMode()
 {
-    assert( ( control & CONTROL_HUMAN ) == CONTROL_HUMAN );
+    assert( ( _control & CONTROL_HUMAN ) == CONTROL_HUMAN );
 
     // If this method has been called, then this mode should be actually enabled
     assert( _isAIAutoControlMode );
@@ -269,7 +249,7 @@ OStreamBase & operator<<( OStreamBase & stream, const Player & player )
 {
     const BitModes & modes = player;
 
-    return stream << modes << player.control << player.color << player.race << player.friends << player.name << player.focus << player._aiPersonality
+    return stream << modes << player._control << player._color << player._race << player._friendsColors << player._name << player._focus << player._aiPersonality
                   << player._handicapStatus;
 }
 
@@ -285,17 +265,23 @@ IStreamBase & operator>>( IStreamBase & stream, Player & player )
         stream >> temp;
     }
 
-    return stream >> player.control >> player.color >> player.race >> player.friends >> player.name >> player.focus >> player._aiPersonality >> player._handicapStatus;
-}
+    stream >> player._control;
 
-Players::Players()
-{
-    reserve( maxNumOfPlayers );
-}
+    static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1109_RELEASE, "Remove the logic below." );
+    if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1109_RELEASE ) {
+        int playerColor;
+        int friendsColors;
 
-Players::~Players()
-{
-    clear();
+        stream >> playerColor >> player._race >> friendsColors;
+
+        player._color = static_cast<PlayerColor>( playerColor );
+        player._friendsColors = static_cast<uint8_t>( friendsColors );
+    }
+    else {
+        stream >> player._color >> player._race >> player._friendsColors;
+    }
+
+    return stream >> player._name >> player._focus >> player._aiPersonality >> player._handicapStatus;
 }
 
 void Players::clear()
@@ -304,21 +290,21 @@ void Players::clear()
 
     std::vector<Player *>::clear();
 
-    _currentColor = Color::NONE;
+    _currentColor = PlayerColor::NONE;
 
     playersArray = {};
-    humanColors = Color::NONE;
+    humanColors = 0;
 }
 
-void Players::Init( int colors )
+void Players::Init( const PlayerColorsSet colors )
 {
     clear();
 
-    const Colors vcolors( colors );
+    const PlayerColorsVector vcolors( colors );
 
-    for ( Colors::const_iterator it = vcolors.begin(); it != vcolors.end(); ++it ) {
-        push_back( new Player( *it ) );
-        playersArray[Color::GetIndex( *it )] = back();
+    for ( const PlayerColor color : vcolors ) {
+        push_back( new Player( color ) );
+        playersArray[Color::GetIndex( color )] = back();
     }
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, "Players: " << String() )
@@ -332,93 +318,85 @@ void Players::Init( const Maps::FileInfo & fi )
     }
 
     clear();
-    const Colors vcolors( fi.kingdomColors );
+    const PlayerColorsVector vcolors( fi.kingdomColors );
 
     Player * first = nullptr;
 
-    for ( const int color : vcolors ) {
+    for ( const PlayerColor color : vcolors ) {
         Player * player = new Player( color );
         player->SetRace( fi.KingdomRace( color ) );
         player->SetControl( CONTROL_AI );
-        player->SetFriends( color | fi.unions[Color::GetIndex( color )] );
+        player->SetFriends( fi.unions[Color::GetIndex( color )] | color );
 
-        if ( ( color & fi.HumanOnlyColors() ) && Settings::Get().IsGameType( Game::TYPE_MULTI ) )
+        if ( ( fi.HumanOnlyColors() & color ) && Settings::Get().IsGameType( Game::TYPE_MULTI ) ) {
             player->SetControl( CONTROL_HUMAN );
-        else if ( color & fi.colorsAvailableForHumans )
+        }
+        else if ( fi.colorsAvailableForHumans & color ) {
             player->SetControl( player->GetControl() | CONTROL_HUMAN );
+        }
 
-        if ( !first && ( player->GetControl() & CONTROL_HUMAN ) )
+        if ( !first && ( player->GetControl() & CONTROL_HUMAN ) ) {
             first = player;
+        }
 
         push_back( player );
         playersArray[Color::GetIndex( color )] = back();
     }
 
-    if ( first )
+    if ( first ) {
         first->SetControl( CONTROL_HUMAN );
+    }
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, "Players: " << String() )
 }
 
-void Players::Set( const int color, Player * player )
+void Players::Set( const PlayerColor color, Player * player )
 {
     playersArray[Color::GetIndex( color )] = player;
 }
 
-Player * Players::Get( int color )
+Player * Players::Get( const PlayerColor color )
 {
     return playersArray[Color::GetIndex( color )];
 }
 
-bool Players::isFriends( int player, int colors )
+bool Players::isFriends( const PlayerColor playerColor, PlayerColorsSet colors )
 {
-    const Player * ptr = Get( player );
-    return ptr ? ( ptr->GetFriends() & colors ) != 0 : false;
+    const Player * player = Get( playerColor );
+    return player ? ( player->GetFriends() & colors ) : false;
 }
 
-void Players::SetPlayerRace( int color, int race )
+void Players::SetPlayerRace( const PlayerColor color, const int race )
 {
     Player * player = Get( color );
 
-    if ( player )
+    if ( player ) {
         player->SetRace( race );
+    }
 }
 
-void Players::SetPlayerControl( int color, int ctrl )
+void Players::SetPlayerControl( const PlayerColor color, const int control )
 {
     Player * player = Get( color );
 
-    if ( player )
-        player->SetControl( ctrl );
+    if ( player ) {
+        player->SetControl( control );
+    }
 }
 
-int Players::GetColors( int control, bool strong ) const
+PlayerColorsSet Players::GetColors( const int control, const bool strong ) const
 {
-    int res = 0;
+    PlayerColorsSet res = 0;
 
-    for ( const_iterator it = begin(); it != end(); ++it )
-        if ( control == 0xFF || ( strong && ( *it )->GetControl() == control ) || ( !strong && ( ( *it )->GetControl() & control ) ) )
-            res |= ( *it )->GetColor();
+    for ( const Player * player : *this ) {
+        if ( control == 0xFF || ( strong && player->GetControl() == control ) || ( !strong && ( player->GetControl() & control ) ) ) {
+            res |= player->GetColor();
+        }
+    }
 
     return res;
 }
 
-int Players::GetActualColors() const
-{
-    int res = 0;
-
-    for ( const_iterator it = begin(); it != end(); ++it )
-        if ( ( *it )->isPlay() )
-            res |= ( *it )->GetColor();
-
-    return res;
-}
-
-const std::vector<Player *> & Players::getVector() const
-{
-    return *this;
-}
-
 Player * Players::GetCurrent()
 {
     return Get( _currentColor );
@@ -429,45 +407,58 @@ const Player * Players::GetCurrent() const
     return Get( _currentColor );
 }
 
-int Players::GetPlayerFriends( int color )
+PlayerColorsSet Players::GetActualColors() const
+{
+    PlayerColorsSet res = 0;
+
+    for ( const Player * player : *this ) {
+        if ( player->isPlay() ) {
+            res |= player->GetColor();
+        }
+    }
+
+    return res;
+}
+
+PlayerColorsSet Players::GetPlayerFriends( const PlayerColor color )
 {
     const Player * player = Get( color );
     return player ? player->GetFriends() : 0;
 }
 
-int Players::GetPlayerControl( int color )
+int Players::GetPlayerControl( const PlayerColor color )
 {
     const Player * player = Get( color );
     return player ? player->GetControl() : CONTROL_NONE;
 }
 
-int Players::GetPlayerRace( int color )
+int Players::GetPlayerRace( const PlayerColor color )
 {
     const Player * player = Get( color );
     return player ? player->GetRace() : Race::NONE;
 }
 
-bool Players::GetPlayerInGame( int color )
+bool Players::GetPlayerInGame( const PlayerColor color )
 {
     const Player * player = Get( color );
     return player && player->isPlay();
 }
 
-std::vector<int> Players::getInPlayOpponents( const int color )
+std::vector<PlayerColor> Players::getInPlayOpponents( const PlayerColor color )
 {
-    std::vector<int> opponentColors;
+    std::vector<PlayerColor> opponentColors;
 
     const Player * playerOfColor = Players::Get( color );
     assert( playerOfColor != nullptr );
 
-    const int friends = playerOfColor->GetFriends();
+    const PlayerColorsSet friends = playerOfColor->GetFriends();
 
     for ( const Player * player : Settings::Get().GetPlayers() ) {
         assert( player != nullptr );
 
-        const int currentColor = player->GetColor();
+        const PlayerColor currentColor = player->GetColor();
 
-        if ( player->isPlay() && ( ( currentColor & friends ) == 0 ) ) {
+        if ( player->isPlay() && ( ( friends & currentColor ) == 0 ) ) {
             opponentColors.emplace_back( currentColor );
         }
     }
@@ -475,14 +466,14 @@ std::vector<int> Players::getInPlayOpponents( const int color )
     return opponentColors;
 }
 
-void Players::SetPlayerInGame( int color, bool f )
+void Players::SetPlayerInGame( const PlayerColor color, bool isPlay )
 {
     Player * player = Get( color );
     if ( player == nullptr ) {
         return;
     }
 
-    player->SetPlay( f );
+    player->SetPlay( isPlay );
 }
 
 void Players::SetStartGame()
@@ -495,22 +486,22 @@ void Players::SetStartGame()
     for_each( begin(), end(), [&races]( Player * player ) { fixRandomRace( player, races ); } );
     for_each( begin(), end(), []( Player * player ) { fixMultiControl( player ); } );
 
-    _currentColor = Color::NONE;
-    humanColors = Color::NONE;
+    _currentColor = PlayerColor::NONE;
+    humanColors = 0;
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, String() )
 }
 
-int Players::HumanColors()
+PlayerColorsSet Players::HumanColors()
 {
-    if ( humanColors == Color::NONE ) {
+    if ( humanColors == 0 ) {
         humanColors = Settings::Get().GetPlayers().GetColors( CONTROL_HUMAN, true );
     }
 
     return humanColors;
 }
 
-int Players::FriendColors()
+PlayerColorsSet Players::FriendColors()
 {
     const Player * player = Settings::Get().GetPlayers().GetCurrent();
     if ( player ) {
@@ -520,10 +511,10 @@ int Players::FriendColors()
     return 0;
 }
 
-void Players::setCurrentColor( const int color )
+void Players::setCurrentColor( const PlayerColor color )
 {
     // We can set only one of 6 player colors ( BLUE | GREEN | RED | YELLOW | ORANGE | PURPLE ) or NONE (neutral player).
-    assert( Color::Count( color ) == 1 || color == Color::NONE );
+    assert( Color::Count( static_cast<PlayerColorsSet>( color ) ) == 1 || color == PlayerColor::NONE );
 
     _currentColor = color;
 }
@@ -563,6 +554,7 @@ std::string Players::String() const
 
 OStreamBase & operator<<( OStreamBase & stream, const Players & players )
 {
+    static_assert( std::is_same_v<PlayerColorsSet, uint8_t> );
     stream << players.GetColors() << players.getCurrentColor();
 
     std::for_each( players.begin(), players.end(), [&stream]( const Player * player ) {
@@ -576,15 +568,29 @@ OStreamBase & operator<<( OStreamBase & stream, const Players & players )
 
 IStreamBase & operator>>( IStreamBase & stream, Players & players )
 {
-    int colors{ 0 };
-    int current{ 0 };
-    stream >> colors >> current;
+    static_assert( std::is_same_v<PlayerColorsSet, uint8_t> );
+    PlayerColorsSet colors;
+    PlayerColor current;
+
+    static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1109_RELEASE, "Remove the logic below." );
+    if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1109_RELEASE ) {
+        int colorsTemp{ 0 };
+        int currentTemp{ 0 };
+
+        stream >> colorsTemp >> currentTemp;
+
+        colors = static_cast<PlayerColorsSet>( colorsTemp );
+        current = static_cast<PlayerColor>( currentTemp );
+    }
+    else {
+        stream >> colors >> current;
+    }
 
     players.clear();
     players.setCurrentColor( current );
 
-    const Colors vcolors( colors );
-    std::for_each( vcolors.begin(), vcolors.end(), [&stream, &players]( const int /* color */ ) {
+    const PlayerColorsVector vcolors( colors );
+    std::for_each( vcolors.begin(), vcolors.end(), [&stream, &players]( const PlayerColor /* color */ ) {
         Player * player = new Player();
         stream >> *player;
 
